@@ -127,7 +127,7 @@ GnomeUIInfo help_menu[] = {
 };
 
 GnomeUIInfo main_menu[] = {
-        { GNOME_APP_UI_SUBTREE_STOCK, N_("_Log"), NULL,
+        { GNOME_APP_UI_SUBTREE, N_("_Log"), NULL,
           log_menu, NULL, NULL, (GnomeUIPixmapType) 0,
           NULL, 0, (GdkModifierType) 0, NULL },
 	GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
@@ -714,6 +714,7 @@ FileSelectOk (GtkWidget * w, GtkFileSelection * fs)
 {
    char *f;
    Log *tl;
+   gint i;
 
    /* Check that we haven't opened all logfiles allowed    */
    if (numlogs >= MAX_NUM_LOGS)
@@ -725,6 +726,17 @@ FileSelectOk (GtkWidget * w, GtkFileSelection * fs)
    f = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
    gtk_widget_destroy (GTK_WIDGET (fs));
 
+   /* Check whether we are opening the already opened log file */ 
+   for ( i = 0; i < numlogs; i++)
+   {
+      if (strcmp (f, loglist[i]->name) == 0)
+      {
+         ShowErrMessage (_("File already opened"));
+         g_free (f);
+         return;
+      }
+   }      
+
    if (f != NULL) {
       if ((tl = OpenLogFile (f)) != NULL)
       {
